# Home

Coluna.jl solves Mixed Integer Programs (MIP) by applying reformulation
techniques such as Dantzig-Wolfe decomposition.
The reformulated problem is solved using a branch-and-price-and-cut
(column and row generation) algorithm. The specificity of Coluna.jl
is to offer a “black-box” implementation of the method:

1. the input is the set of constraints and variables of the MIP in
   its natural/compact formulation (formulated with JuMP or MOI); the Block-Decompostion.jl package offers extensions to the modeling language JuMP to allow the user to provide instructions on decomposition to Coluna;

2. the user defines subsystems in the MIP on which the decomposition is based; subsystems are described by rows and/or columns indices; this is handy to
   test different decompositions;

3. the reformulation is automatically generated by Coluna,
   without any input from the user to define master columns,
   their reduced cost, pricing/separation problem, or Lagrangian bound;

4. a default column and cut generation procedure is implemented. It relies
   on underlying MOI optimizers to handle master and subproblem
   but the user can define its own optimizer;

5. a branching scheme that preserves the pricing problem structure
   is offered by default; it runs based on priorities and directives
   specified by the user on the original variables; default primal heuristics
   and preprocessing features are under developments.

The user must be familiar with the syntax of JuMP, which is described in its
[documentation](http://www.juliaopt.org/JuMP.jl/v0.19.0/).

## Manual Outline

```@contents
Pages = [
    "index.md",
    "installation.md",
    "start.md"
]
Depth = 1
```    
